package de.fzj.unicore.ucc.admin;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.cli.OptionBuilder;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.TargetSystemFactory;
import de.fzj.unicore.ucc.actions.ActionBase;
import de.fzj.unicore.wsrflite.xmlbeans.client.AdminServiceClient;
import de.fzj.unicore.wsrflite.xmlbeans.client.AdminServiceClient.AdminActionResult;

public class RunCommand extends ActionBase{

	public static final String OPT_URL_LONG="url";
	public static final String OPT_URL="u";

	private String siteName;
	private String url;

	private String cmd;
	private final Map<String,String>params=new HashMap<String,String>();

	@Override
	@SuppressWarnings("all")
	protected void createOptions() {
		super.createOptions();
		getOptions().addOption(OptionBuilder.withLongOpt(OPT_SITENAME_LONG)
				.withDescription("Site Name")
				.withArgName("Vsite")
				.hasArg()
				.isRequired(false)
				.create(OPT_SITENAME)
				);
		getOptions().addOption(OptionBuilder.withLongOpt(OPT_URL_LONG)
				.withDescription("Admin service URL")
				.withArgName("URL")
				.hasArg()
				.isRequired(false)
				.create(OPT_URL)
				);
	}

	@Override
	public void process() {
		super.process();
		int length=getCommandLine().getArgs().length;

		if(length<2){
			throw new IllegalArgumentException("You must provide at least a command name as argument.");
		}
		params.clear();
		cmd=getCommandLine().getArgs()[1];

		for(int i=2; i<length; i++){
			String p=getCommandLine().getArgs()[i];
			String[]split=p.split("=");
			String key=split[0];
			String value=split[1];
			verbose("Have parameter: "+key+"="+value);
			params.put(key, value);
		}

		siteName=getCommandLine().getOptionValue(OPT_SITENAME);
		url=getCommandLine().getOptionValue(OPT_URL);
		if(siteName==null && url==null){
			//try to guess URL from the registry URL
			if(registryURL!=null){
				try{
					String regurl=registryURL.split("[, ]")[0];
					url=regurl.replace("Registry?res=default_registry", "AdminService?res=default_admin");
					verbose("Will try fallback admin service URL: "+url);
				}catch(Exception ex){}
			}
			if(url==null){
				error("Either URL or site name must be given!",null);
				endProcessing(ERROR_CLIENT);
			}
		}
		if(siteName!=null && url!=null){
			error("URL and site name cannot both be given!",null);
			endProcessing(ERROR_CLIENT);
		}

		AdminServiceClient asc=null;
		List<AdminServiceClient.AdminAction>availableCmds=null;
		try{
			asc=createClient();
			//check access
			asc.getCurrentTime();
			verbose("Contacted admin service at <"+asc.getUrl()+">");
			availableCmds=asc.getAdminActions();
		}catch(Exception ex){
			error("Error contacting admin service at <"+asc.getUrl()+">",ex);
			endProcessing(1);
		}
		//check command availability
		if(!availableCmds.contains(new AdminServiceClient.AdminAction(cmd,null))){
			error("No such command: <"+cmd+">",null);
			endProcessing(ERROR_CLIENT);
		}
		AdminActionResult aar=null;
		try{
			aar=asc.invokeAdminAction(cmd, params);
			if(aar.successful()){
				message("SUCCESS, service reply: "+aar.getMessage());
				if(aar.getResults().size()>0){
					message(String.valueOf(aar.getResults()));
				}
			}else{
				message("Action was NOT SUCCESSFUL, service reply: "+aar.getMessage());
			}
		}catch(Exception ex){
			error("Error executing admin command",ex);
			endProcessing(1);
		}

	}

	private AdminServiceClient createClient()throws Exception{
		if(url==null){
			findURL();
		}
		EndpointReferenceType epr=EndpointReferenceType.Factory.newInstance();
		epr.addNewAddress().setStringValue(url);
		AdminServiceClient asc=new AdminServiceClient(epr,securityProperties);
		return asc;
	}
	private void findURL()throws Exception{
		List<EndpointReferenceType>tsfs=registry.listServices(TargetSystemFactory.TSF_PORT);
		EndpointReferenceType epr=EndpointReferenceType.Factory.newInstance();
		String adminServiceURL=null;
		for(EndpointReferenceType e: tsfs){
			String u=e.getAddress().getStringValue();
			if(u.contains("/"+siteName+"/")){
				epr=e;
				break;	
			}
		}
		if(epr==null){
			throw new IllegalArgumentException("No matching site found.");
		}
		String tsfURL=epr.getAddress().getStringValue();
		int endIndex=tsfURL.lastIndexOf("TargetSystemFactoryService");
		adminServiceURL=epr.getAddress().getStringValue().substring(0, endIndex)+"AdminService?res=default_admin";
		url=adminServiceURL;
	}

	@Override
	public String getName() {
		return "admin-runcommand";
	}

	@Override
	public String getArgumentList() {
		return "<command> [key1=value1 key2=value2 ...]";
	}

	@Override
	public String getSynopsis() {
		return "Runs a server-side administrative command, which can be parametrised, " 
				+"and displays the results. The mandatory '-s' option is used to select the site. "
				+"";
	}

	@Override
	public String getDescription() {
		return "Run a server-side administrative command";
	}

	@Override
	public String getCommandGroup() {
		return AdminCommands.CMD_GROUP;
	}

}
