/*
 * Copyright (c) 2011 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on 26-05-2011
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */
package eu.unicore.ucc.ccm;

import java.io.File;
import java.io.FileReader;
import java.util.Properties;

import junit.framework.Assert;

import org.junit.Test;

import de.fzj.unicore.ucc.Constants;
import eu.unicore.util.httpclient.ClientProperties;

import xmlbeans.eu.unicore.ccm.internal.ClientProfileDocument.ClientProfile;
import xmlbeans.eu.unicore.ccm.internal.ClientPropertiesType;
import xmlbeans.eu.unicore.ccm.internal.PropertyExtType;
import xmlbeans.eu.unicore.ccm.internal.RegistryType;

public class PropertiesUpdaterTest
{
	@Test
	public void testNewRmDefault1()
	{
		final String cfg = "src/test/resources/client1.properties";
		ClientProfile profile = ClientProfile.Factory.newInstance();
		RegistryType reg = profile.addNewRegistry();
		reg.setAddress("https://127.0.0.1");
		reg.setDescription("New reg");
		reg.setSuggestedLabel("newLabel");
		profile.setRemoveDefaults(true);
		
		String file = performUpdate(cfg, profile, false);
		testProperty(Constants.OPT_REGISTRY_LONG, "https://127.0.0.1", file);
	}

	
	@Test
	public void testUpdateRmDefault1()
	{
		final String cfg = "src/test/resources/client1.properties";
		ClientProfile profile = ClientProfile.Factory.newInstance();
		RegistryType reg = profile.addNewRegistry();
		reg.setAddress("https://127.0.0.1");
		reg.setDescription("New reg");
		reg.setSuggestedLabel("newLabel");
		profile.setRemoveDefaults(true);
		
		String file = performUpdate(cfg, profile, true);
		testProperty(Constants.OPT_REGISTRY_LONG, "https://127.0.0.1", file);
	}

	@Test
	public void testUpdate2()
	{
		final String cfg = "src/test/resources/client1.properties";
		ClientProfile profile = ClientProfile.Factory.newInstance();
		RegistryType reg = profile.addNewRegistry();
		reg.setAddress("https://127.0.0.1/new.registry");
		profile.setRemoveDefaults(false);
		
		String file = performUpdate(cfg, profile, true);
		testProperty(Constants.OPT_REGISTRY_LONG, 
				"https://localhost:8080/DEMO-SITE/services/Registry?res=default_registry https://127.0.0.1/new.registry", 
				file);
	}

	@Test
	public void testUpdate3()
	{
		final String cfg = "src/test/resources/client2.properties";
		ClientProfile profile = ClientProfile.Factory.newInstance();
		RegistryType reg = profile.addNewRegistry();
		reg.setAddress("https://127.0.0.1/new.registry");
		RegistryType reg2 = profile.addNewRegistry();
		reg2.setAddress("https://127.0.0.1:8080/SITE/services/Registry?res=some_res");
		RegistryType reg3 = profile.addNewRegistry();
		reg3.setAddress("https://127.0.0.1/new.registry3");
		RegistryType reg4 = profile.addNewRegistry();
		reg4.setAddress("https://127.0.0.1:8080/SOME-SITE/services/Registry?res=default_registry");
		profile.setRemoveDefaults(true);
		
		String file = performUpdate(cfg, profile, true);
		testProperty(Constants.OPT_REGISTRY_LONG, 
				"https://localhost:8080/SOME-SITE/services/Registry?res=default_registry https://127.0.0.1:8989/SOME-SITE2/services/Registry?res=default_registry " +
				"https://127.0.0.1/new.registry https://127.0.0.1:8080/SITE/services/Registry?res=some_res https://127.0.0.1/new.registry3", 
				file);
	}
	
	@Test
	public void testGenericUpdate()
	{
		final String cfg = "src/test/resources/client2.properties";
		ClientProfile profile = ClientProfile.Factory.newInstance();
		ClientPropertiesType cliProps = profile.addNewClientProperties();
		PropertyExtType prop = cliProps.addNewProperty();
		prop.setName("output");
		prop.setOverwrite(true);
		prop.setStringValue("FOO");
		
		prop = cliProps.addNewProperty();
		prop.setName("password");
		prop.setOverwrite(false);
		prop.setStringValue("BAR");
		
		prop = cliProps.addNewProperty();
		prop.setName(ClientProperties.DEFAULT_PREFIX+ClientProperties.PROP_HTTP_USER);
		prop.setStringValue("BAZ");
		
		String file = performUpdate(cfg, profile, true);
		testProperty("output", "FOO", file);
		testProperty("password", "the!user", file);
		testProperty(ClientProperties.DEFAULT_PREFIX+ClientProperties.PROP_HTTP_USER, "BAZ", file);
	}
	
	private void testProperty(String property, String value, String file)
	{
		Properties p = new Properties();
		try
		{
			p.load(new FileReader(file));
		} catch (Exception e)
		{
			e.printStackTrace();
			Assert.fail(e.toString());
		}
		String real = p.getProperty(property);
		Assert.assertEquals("Got incorrect property value", value, real);
	}
	
	private String performUpdate(String cfg, ClientProfile profile, boolean update)
	{
		File workdir = Utils.createWorkdirWithFile(cfg);
		File newDir = Utils.createWorkdirForUpdate("new-configs", workdir, update);
		
		PropertiesUpdater updater = new PropertiesUpdater(
				new MockMessageWriter(), 
				newDir.getAbsolutePath());
		File workFile = new File(workdir, new File(cfg).getName());
		try
		{
			updater.processRegistries(profile, workFile.getAbsolutePath(), update);
		} catch (CCMException e)
		{
			e.printStackTrace();
			Assert.fail(e.toString());
		}
		if (update)
			return workFile.toString();
		else
			return new File(newDir, new File(cfg).getName()).toString();
	}
}
