package de.fzj.unicore.ucc.actions;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStreamReader;

import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.client.JobClient;
import de.fzj.unicore.ucc.util.Builder;

public class AbortJob extends ActionBase {

	private Builder builder;
	private JobClient job;

	protected void initBuilder(){
		String arg=null;
		if(getCommandLine().getArgs().length>1)arg=getCommandLine().getArgs()[1];
		
		String url=null;
		if(arg==null){
			try{
				arg=new BufferedReader(new InputStreamReader(System.in)).readLine();
			}catch(Exception e){
				System.err.println("Can't read job descriptor from stdin.");
				endProcessing(ERROR_CLIENT);
			}
		}
		

		initFromCmdline(arg);
		url=builder.getProperty("epr");
		if(url==null){
			System.err.println("Job EPR not found! Maybe <"+arg+"> has not been produced by ucc.");
			endProcessing(ERROR_CLIENT);
		}

		try{
			EndpointReferenceType epr=EndpointReferenceType.Factory.newInstance();
			epr.addNewAddress().setStringValue(url);
			job = new JobClient(url,epr,securityProperties);
			job.getCurrentTime();
			verbose("Job ID OK.");
		}
		catch(Exception e){
			error("Can't contact job", e);
			endProcessing(ERROR);
		}
	}

	private void initFromCmdline(String arg){
		try{
			File job=new File(arg);
			if(job.exists())
			{
				builder=new Builder(job);
				verbose("Read job from <"+arg+">");
			}
			else{
				verbose("Accessing job at EPR = <"+arg+">");
				builder=new Builder();
				builder.setProperty("epr", arg);
			}
		}catch(Exception e){
			error("Can't use <"+arg+">.",e);
			endProcessing(ERROR_CLIENT);
		}
				
	}

	@Override
	public void process(){
		super.process();
		initBuilder();
		abort();
	}
	
	@Override
	public String getName(){
		return "abort-job";
	}
	
	@Override
	public String getArgumentList(){
		return "[<jobfile> | <jobepr>]";
	}
	@Override
	public String getSynopsis(){
		return "Aborts a UNICORE 6 job. " +
				"The job is referenced either by a job file as written " +
				"by the 'run' command or as an EPR.";
	}
	@Override
	public String getDescription(){
		return "abort a job";
	}
	@Override
	public String getCommandGroup(){
		return "Job execution";
	}
	
	
	
	protected void abort(){
		try{
			verbose("Job id: " +job.getEPR().getAddress().getStringValue());
			job.abort();
			message("Job aborted.");
		}catch(Exception e){
			error("Can't abort job.",e);
			endProcessing(ERROR);
		}
	}
	
}
