package de.fzj.unicore.ucc.actions;

import java.io.OutputStream;

import org.unigrids.services.atomic.types.ProtocolType;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.client.HttpBasicClient;
import de.fzj.unicore.uas.client.StorageClient;
import de.fzj.unicore.uas.fts.FiletransferOptions.SupportsPartialRead;
import de.fzj.unicore.ucc.MessageWriter;
import de.fzj.unicore.ucc.StorageConstants;
import de.fzj.unicore.ucc.helpers.Location;
import de.fzj.unicore.ucc.util.FileDownloader;
import de.fzj.unicore.ucc.util.Mode;
import de.fzj.unicore.wsrflite.xmlbeans.WSUtilities;
import de.fzj.unicore.wsrflite.xmlbeans.client.IRegistryQuery;
import eu.unicore.util.httpclient.IClientConfiguration;

/**
 * cat a file from a remote sms
 */
public class CatFile extends FileOperation implements StorageConstants {

	protected Location sourceDesc;
	protected StorageClient sms;

	@Override
	@SuppressWarnings("all")
	protected void createOptions() {
		super.createOptions();

	}

	@Override
	protected boolean needSourceTargetOpts(){
		return false;
	}

	@Override
	public void process() {
		super.process();

		if(getCommandLine().getArgs().length>1){
			for(int i=1; i<getCommandLine().getArgs().length;i++){
				source=getCommandLine().getArgs()[1];
				sourceDesc=resolve(source,registry,securityProperties,this);
				try{
					if(sourceDesc.isRaw()){
						verbose("Source file URL "+sourceDesc.getSmsEpr());
						runRawTransfer();
					}
					else{
						verbose("Source file URL "+sourceDesc.getUnicore6URI());
						runFileDownloader();
					}
				}catch(Exception e){
					error("Can't get file.",e);
					endProcessing(ERROR);
				}
				System.out.println();
			}
		}
		else{
			error("Please specify a remote file!",null);
			endProcessing(ERROR);
		}
	}

	/**
	 * resolve a location
	 * 
	 * @param uri
	 * @param registry
	 * @param msg
	 * @return {@link Location} corresponding to the given URI
	 */
	protected Location resolve(String uri, IRegistryQuery registry, IClientConfiguration sec,MessageWriter msg){
		return Resolve.resolve(uri,registry,sec,msg);
	}


	private void runFileDownloader()throws Exception{
		EndpointReferenceType e=EndpointReferenceType.Factory.newInstance();
		e.addNewAddress().setStringValue(sourceDesc.getSmsEpr());
		String dn=findServerName(e);
		if(dn!=null)WSUtilities.addServerIdentity(e, dn);
		sms=new StorageClient(sourceDesc.getSmsEpr(),e,securityProperties);
		verbose("SMS contacted, server reply: servertime="+sms.getCurrentTime());

		FileDownloader exp=new FileDownloader(sourceDesc.getName(),"",Mode.overwrite);
		String sourceDescProtocol=sourceDesc.getProtocol();
		if(!"BFT".equalsIgnoreCase(sourceDescProtocol)){
			try{
				preferredProtocols.add(0, ProtocolType.Enum.forString(sourceDescProtocol));
			}catch(Exception ignored){}
		}
		exp.setPreferredProtocols(preferredProtocols);
		exp.setExtraParameterSource(properties);
		exp.setTiming(timing);
		exp.setForceFileOnly(true);
		exp.setShowProgress(false);
		exp.setTargetStream(getStdout());
		if(startByte!=null){
			exp.setStartByte(startByte);
			exp.setEndByte(endByte);
		}
		exp.perform(sms, this);
	}

	private void runRawTransfer()throws Exception {
		String url=sourceDesc.getSmsEpr();
		//TODO nicer way to find and configure protocol handlers
		if(url.startsWith("http")){
			HttpBasicClient hc=new HttpBasicClient(url, securityProperties);
			OutputStream os=getStdout();
			try{
				if(startByte!=null){
					verbose("Byte range: "+startByte+"-"+endByte);
					SupportsPartialRead pReader=(SupportsPartialRead)hc;
					pReader.readPartial(startByte, endByte-startByte, os);
				}
				else{
					hc.readAllData(os);
				}
			}finally{
				os.close();
			}
		}
		else throw new Exception("No protocol handler for "+url);
	}

	OutputStream getStdout(){
		return System.out;
	}

	@Override
	public String getName() {
		return "cat";
	}

	@Override
	public String getSynopsis() {
		return "Prints a file from remote location to stdout";
	}
	@Override
	public String getDescription(){
		return "cat remote files";
	}

	@Override
	public String getArgumentList(){
		return "<file_url>";
	}
	@Override
	public String getCommandGroup(){
		return "Data management";
	}

}
