/*
 * Copyright (c) 2011 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on 27-07-2011
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */
package de.fzj.unicore.ucc.actions;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.OptionBuilder;

import de.fzj.unicore.uas.client.StorageClient;

/**
 * Changes ownership (Xlogin/Xgroup) of a given file.
 * 
 * @author K. Benedyczak
 */
public class Chgrp extends SMSOperation {

	@Override
	@SuppressWarnings("all")
	protected void createOptions() {
		super.createOptions();
		getOptions().addOption(OptionBuilder.withLongOpt(OPT_RECURSIVE_LONG)
				.withDescription("Recursively applies the operation to the whole contents of the direcotry specified as an argument.")
				.isRequired(false)
				.create(OPT_RECURSIVE)
				);
	}

	@Override
	public String getName() {
		return "chgrp";
	}

	@Override
	public String getSynopsis() {
		return "Changes an owning group of a file on a remote storage.";
	}
	
	@Override
	public String getDescription(){
		return "change file's group";
	}
	
	@Override
	public String getArgumentList(){
		return "<group> <[Storage-URL#]/path or [u6://SITENAME/[JobId|StorageName]]/path>";
	}
	
	@Override
	public String getCommandGroup(){
		return "Data management";
	}
	
	
	@Override
	public void process() {
		super.process();
		boolean recursive = getBooleanOption(OPT_RECURSIVE_LONG, OPT_RECURSIVE);
		verbose("Recursive invocation = " + recursive);
		CommandLine cmdLine = getCommandLine(); 
		if (cmdLine.getArgs().length != 3) {
			error("Not enough arguments", null);
			printUsage();
			endProcessing(ERROR_CLIENT);
		}
		String group = cmdLine.getArgs()[1];
		String target = cmdLine.getArgs()[2];
		
		verbose("Setting group to " + group + " on " + target);
		StorageClient sms = getStorageClient(target);
		
		try{
			String path = getPathAtStorage(target);
			
			sms.chgrp(path, group, recursive);
		}catch(Exception ex){
			error("Can't change file group.",ex);
			endProcessing(ERROR);
		}
	}
}





