package de.fzj.unicore.ucc.actions;

import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStream;

import org.apache.commons.cli.OptionBuilder;
import org.unigrids.services.atomic.types.ProtocolType;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.client.HttpBasicClient;
import de.fzj.unicore.uas.client.StorageClient;
import de.fzj.unicore.uas.fts.FiletransferOptions.SupportsPartialRead;
import de.fzj.unicore.ucc.MessageWriter;
import de.fzj.unicore.ucc.helpers.Location;
import de.fzj.unicore.ucc.util.FileDownloader;
import de.fzj.unicore.ucc.util.Mode;
import de.fzj.unicore.ucc.util.ProgressBar;
import de.fzj.unicore.wsrflite.xmlbeans.WSUtilities;
import de.fzj.unicore.wsrflite.xmlbeans.client.IRegistryQuery;
import eu.unicore.util.httpclient.IClientConfiguration;

/**
 * get a file from a remote sms
 */
public class GetFile extends FileOperation {

	protected Location sourceDesc;
	protected StorageClient sms;
	protected boolean append;
	protected boolean recurse;

	@Override
	@SuppressWarnings("all")
	protected void createOptions() {
		super.createOptions();
		getOptions().addOption(OptionBuilder.withLongOpt(OPT_APPEND_LONG)
				.withDescription("Append to (existing) file(s)")
				.isRequired(false)
				.create(OPT_APPEND)
				);
		getOptions().addOption(OptionBuilder.withLongOpt(OPT_RECURSIVE_LONG)
				.withDescription("Recurse into subdirectories")
				.isRequired(false)
				.create(OPT_RECURSIVE)
				);
	}

	@Override
	public void process() {
		super.process();
		sourceDesc=resolve(source,registry,securityProperties,this);
		append=getBooleanOption(OPT_APPEND_LONG, OPT_APPEND);
		verbose("Appending data="+append);
		recurse=getBooleanOption(OPT_RECURSIVE_LONG, OPT_RECURSIVE);
		if(recurse)verbose("Recurse into subdirectories="+recurse);
		try{
			if(sourceDesc.isRaw()){
				verbose("Source file URL "+sourceDesc.getSmsEpr());
				runRawTransfer();
			}
			else{
				verbose("Source file URL "+sourceDesc.getUnicore6URI());
				runFileDownloader();
			}
		}catch(Exception e){
			error("Can't get file.",e);
			endProcessing(ERROR);
		}
	}

	/**
	 * resolve a location
	 * 
	 * @param uri
	 * @param registry
	 * @param msg
	 * @return {@link Location} corresponding to the given URI
	 */
	protected Location resolve(String uri, IRegistryQuery registry, IClientConfiguration sec,MessageWriter msg){
		return Resolve.resolve(uri,registry,sec,msg);
	}


	private void runFileDownloader()throws Exception{
		EndpointReferenceType e=EndpointReferenceType.Factory.newInstance();
		e.addNewAddress().setStringValue(sourceDesc.getSmsEpr());
		String dn=findServerName(e);
		if(dn!=null)WSUtilities.addServerIdentity(e, dn);
		sms=new StorageClient(sourceDesc.getSmsEpr(),e,securityProperties);
		verbose("SMS contacted, server reply: servertime="+sms.getCurrentTime());
		Mode mode=append?Mode.append:Mode.overwrite;
		FileDownloader exp=new FileDownloader(sourceDesc.getName(),target,mode);
		String sourceDescProtocol=sourceDesc.getProtocol();
		if(!"BFT".equalsIgnoreCase(sourceDescProtocol)){
			try{
				preferredProtocols.add(0, ProtocolType.Enum.forString(sourceDescProtocol));
			}catch(Exception ignored){}
		}
		exp.setPreferredProtocols(preferredProtocols);
		exp.setExtraParameterSource(properties);
		exp.setTiming(timing);
		exp.setRecurse(recurse);
		if(startByte!=null){
			exp.setStartByte(startByte);
			exp.setEndByte(endByte);
		}
		exp.perform(sms, this);
	}
	
	private void runRawTransfer()throws Exception {
		String url=sourceDesc.getSmsEpr();
		//TODO nicer way to find and configure protocol handlers
		if(url.startsWith("http")){
			HttpBasicClient hc=new HttpBasicClient(url, securityProperties);
			File tFile=new File(target);
			OutputStream os=new FileOutputStream(tFile);
			try{
				ProgressBar p=new ProgressBar(tFile.getName(),-1,this);
				hc.setProgressListener(p);
				if(startByte!=null){
					verbose("Byte range: "+startByte+"-"+endByte);
					SupportsPartialRead pReader=(SupportsPartialRead)hc;
					pReader.readPartial(startByte, endByte-startByte, os);
				}
				else{
					hc.readAllData(os);
				}
				p.finish();
			}finally{
				os.close();
			}
		}
		else throw new Exception("No protocol handler for "+url);
	}

	@Override
	public String getName() {
		return "get-file";
	}

	@Override
	public String getSynopsis() {
		return "Gets a file from remote location '-s' and writes it to the local file specified by the '-t' option.";
	}
	@Override
	public String getDescription(){
		return "get remote files";
	}

	@Override
	public String getArgumentList(){
		return "";
	}
	@Override
	public String getCommandGroup(){
		return "Data management";
	}

}
