/*
 * Copyright (c) 2011 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on 28-07-2011
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */
package de.fzj.unicore.ucc.actions;

import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.client.StorageClient;
import de.fzj.unicore.ucc.helpers.EndProcessingException;
import de.fzj.unicore.ucc.helpers.Location;
import de.fzj.unicore.wsrflite.xmlbeans.WSUtilities;

/**
 * Adds some common methods useful for operations on SMS.
 * @author K. Benedyczak
 */
public abstract class SMSOperation extends ActionBase {

	/**
	 * Gets location an a SMS from the provided URL. See {@link Location} class for supported
	 * parameter formats.
	 * @param target
	 * @return File's path at the storage.
	 */
	protected String getPathAtStorage(String target) {
		Location targetDesc = new Location(target, registry, securityProperties, this);
		return targetDesc.getName();
	}
	
	/**
	 * Creates a {@link StorageClient} for the provided target file. Processing is
	 * ended on error.
	 * @param target
	 * @return StorageClient for the target file 
	 */
	protected StorageClient getStorageClient(String target) {
		Location targetDesc = new Location(target, registry, securityProperties, this);
		EndpointReferenceType e = EndpointReferenceType.Factory.newInstance();
		e.addNewAddress().setStringValue(targetDesc.getSmsEpr());
		try {
			String dn = findServerName(e);
			if (dn != null)
				WSUtilities.addServerIdentity(e, dn);
			StorageClient sms = new StorageClient(targetDesc.getSmsEpr(), e, 
					securityProperties);
			if (isVerbose())
				verbose("SMS " + targetDesc.getSmsEpr()
					+ " contacted, server reply: servertime=" + sms.getCurrentTime());
			
			return sms;
		} catch(Exception ex) {
			error("Can't contact the storage service of " + target, ex);
			endProcessing(ERROR);
			//dummy
			throw new EndProcessingException(ERROR);
		}
	}
}
