package de.fzj.unicore.ucc.actions;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.File;

import org.apache.commons.io.FileUtils;
import org.junit.Test;
import org.oasisOpen.docs.wsrf.rp2.GetResourcePropertyDocumentResponseDocument;
import org.unigrids.x2006.x04.services.jms.JobPropertiesDocument;

import de.fzj.unicore.ucc.UCC;
import de.fzj.unicore.ucc.util.EmbeddedTestBase;
import de.fzj.unicore.wsrflite.xmlbeans.WSUtilities;
import eu.unicore.bugsreporter.annotation.FunctionalTest;

/**
 * Functional tests for the UCC actions. 
 * These run against an embedded UNICORE instance.
 */
public class TestStorageActions extends EmbeddedTestBase {


	@FunctionalTest(id="testMkdirRM", description="Tests the mkdir and rm commands")
	@Test
	public void test_Mkdir_RM(){
		connect();

		String storage=createNewUspace();

		String dir="#y/test"+System.currentTimeMillis();
		String[] args=new String[]{"mkdir", "-v", 
				"-c", "src/test/resources/conf/userprefs.embedded",
				storage+dir,
		};
		UCC.main(args);
		assertEquals(Integer.valueOf(0),UCC.exitCode);

		args=new String[]{"rm", "-v", "-q",
				"-c", "src/test/resources/conf/userprefs.embedded",
				storage+dir,
		};
		UCC.main(args);
		assertEquals(Integer.valueOf(0),UCC.exitCode);
	}


	@FunctionalTest(id="testResolve", description="Tests the resolve command")
	@Test
	public void test_Resolve(){
		connect();
		runDate();
		String id=WSUtilities.extractResourceID(Run.getLastJobAddress());

		String[] args=new String[]{"resolve", "-v",
				"-c", "src/test/resources/conf/userprefs.embedded",
				"u6://UCC-TEST/"+id,
		};
		UCC.main(args);
		assertEquals(Integer.valueOf(0),UCC.exitCode);

		args=new String[]{"resolve", "-v", "-f",
				"-c", "src/test/resources/conf/userprefs.embedded",
				"u6://UCC-TEST/"+id+"/stdout?protocol=UFTP",
		};
		UCC.main(args);
		assertEquals(Integer.valueOf(0),UCC.exitCode);
	}

	@FunctionalTest(id="testMetadata", description="Tests the 'metadata' command")
	@Test
	public void test_Metadata()throws Exception{
		connect();

		String[] args=new String[]{"create-storage", "-v", 
				"-c", "src/test/resources/conf/userprefs.embedded",
		};
		UCC.main(args);
		assertEquals(Integer.valueOf(0),UCC.exitCode);
		String storage=CreateStorage.getLastStorageAddress();

		File metaFile=new File("target", "testmetadata.json");
		FileUtils.writeStringToFile(metaFile, "{foo: bar}");

		args=new String[]{"metadata",
				"-c", "src/test/resources/conf/userprefs.embedded",
				"-s", storage,
				"-C", "create",
				"-f", metaFile.getPath(),
				"testfile",
		};
		UCC.main(args);
		assertEquals(Integer.valueOf(0),UCC.exitCode);

		args=new String[]{"metadata",
				"-c", "src/test/resources/conf/userprefs.embedded",
				"-s", storage,
				"-C", "read",
				"testfile",
		};
		UCC.main(args);
		assertEquals(Integer.valueOf(0),UCC.exitCode);
		assertEquals("bar", Metadata.getLastMeta().get("foo"));

		FileUtils.writeStringToFile(metaFile, "{foo: spam}");
		args=new String[]{"metadata",
				"-c", "src/test/resources/conf/userprefs.embedded",
				"-s", storage,
				"-C", "update",
				"-f", metaFile.getPath(),
				"testfile",
		};
		UCC.main(args);
		assertEquals(Integer.valueOf(0),UCC.exitCode);
		assertEquals("spam", Metadata.getLastMeta().get("foo"));

		FileUtils.writeStringToFile(metaFile, "{foo: spam}");
		args=new String[]{"metadata",
				"-c", "src/test/resources/conf/userprefs.embedded",
				"-s", storage,
				"-C", "delete",
				"testfile",
		};
		UCC.main(args);
		assertEquals(Integer.valueOf(0),UCC.exitCode);

		args=new String[]{"metadata",
				"-c", "src/test/resources/conf/userprefs.embedded",
				"-s", storage,
				"-C", "read",
				"testfile",
		};
		UCC.main(args);
		assertEquals(Integer.valueOf(0),UCC.exitCode);
		assertTrue(!Metadata.getLastMeta().containsKey("foo"));

		args=new String[]{"metadata",
				"-c", "src/test/resources/conf/userprefs.embedded",
				"-s", storage,
				"-C", "start-extract",
				"/",
		};
		UCC.main(args);
		assertEquals(Integer.valueOf(0),UCC.exitCode);
		Thread.sleep(1500);
	}

	@FunctionalTest(id="testGetFileProperties", description="Tests the 'stat' command")
	@Test
	public void test_Stat()throws Exception{
		connect();
		String storage=createNewUspace();

		String[] args=new String[]{"stat",
				"-c", "src/test/resources/conf/userprefs.embedded",
				storage+"#/stdout",

		};
		UCC.main(args);
		assertEquals(Integer.valueOf(0),UCC.exitCode);
	}

	@FunctionalTest(id="testGetUmask", description="Tests the 'umask' command")
	@Test
	public void test_Umask()throws Exception{
		connect();
		String storage=createNewUspace();

		String[] args=new String[]{"umask",
				"-c", "src/test/resources/conf/userprefs.embedded",
				storage,
		};
		UCC.main(args);
		assertEquals(Integer.valueOf(0),UCC.exitCode);

		args=new String[]{"umask",
				"-c", "src/test/resources/conf/userprefs.embedded",
				"-s", "777",
				storage,
		};
		UCC.main(args);
		assertEquals(Integer.valueOf(0),UCC.exitCode);

		args=new String[]{"umask",
				"-c", "src/test/resources/conf/userprefs.embedded",
				storage,
		};
		UCC.main(args);
		assertEquals(Integer.valueOf(0),UCC.exitCode);
	}

	@FunctionalTest(id="testChmod", description="Tests the 'chmod' command")
	@Test
	public void test_Chmod()throws Exception{
		connect();
		String storage=createNewUspace();
		String[] perms={"u=rw-","u+--x",};
		String[] args=null;
		for(String perm: perms){
			args=new String[]{"chmod",
					"-c", "src/test/resources/conf/userprefs.embedded",
					perm,
					storage+"#/stdout",
			};
		}
		UCC.main(args);
		assertEquals(Integer.valueOf(0),UCC.exitCode);
	}

	protected String createNewUspace(){
		String[] args=new String[]{"run",
				"-c", "src/test/resources/conf/userprefs.embedded",
				"src/test/resources/jobs/date.u",
		};
		UCC.main(args);
		assertEquals(Integer.valueOf(0),UCC.exitCode);
		
		args=new String[]{"wsrf", "getproperties", 
				"-c", "src/test/resources/conf/userprefs.embedded",
				Run.getLastJobAddress(),
		};
		UCC.main(args);
		assertEquals(Integer.valueOf(0),UCC.exitCode);
		
		try{
			GetResourcePropertyDocumentResponseDocument d=GetResourcePropertyDocumentResponseDocument.Factory.parse(WSRF.getLastProperties());
			JobPropertiesDocument jd= JobPropertiesDocument.Factory.parse(d.getGetResourcePropertyDocumentResponse().newInputStream());
			return jd.getJobProperties().getWorkingDirectoryReference().getAddress().getStringValue();
		}catch(Exception ex){
			ex.printStackTrace();
			fail(); 
		}
		return null;
	}

}
