package de.fzj.unicore.ucc.helpers;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.io.File;

import org.junit.Test;

import de.fzj.unicore.ucc.MessageWriter;

public class TestTransferdescriptor implements MessageWriter {

	@Test
	public void testUnicoreURL(){
		String u1="SBYTEIO:http://some:1234/services/SMS?id#test.txt";
		Location td=new Location(u1,null,null,this);
		assertEquals("test.txt", td.getName());
		assertEquals("SBYTEIO", td.getProtocol());
		assertEquals("http://some:1234/services/SMS?id", td.getSmsEpr());
		assertFalse(td.isLocal());
		assertTrue(td.isUnicoreURL());
		assertFalse(td.isRaw());
	}
	
	@Test
	public void testLocalURL(){
		File f=new File(System.getProperty("java.io.tmpdir"));
		String u1=f.getAbsolutePath();
		Location td=new Location(u1,null,null,this);
		assertEquals(u1, td.getName());
		assertNull("", td.getProtocol());
		assertNull("", td.getSmsEpr());
		assertTrue(td.isLocal());
		assertFalse(td.isUnicoreURL());
		assertFalse(td.isRaw());
	}
	
	@Test
	public void testUrlPattern(){
		String url="ftp://asdkfjlsdf";
		assertFalse(Location.isU6URL(url));
		assertTrue(Location.isRawURL(url));
		
		url="mailto:someone@somewhere?subject=Test&body=This%20is%20a%20test.";
		assertFalse(Location.isU6URL(url));
		assertTrue(Location.isRawURL(url));
		
		url="a\\:test";
		assertFalse(Location.isU6URL(url));
		assertFalse(Location.isRawURL(url));
		
		url="scp://asdkfjlsdf";
		assertFalse(Location.isU6URL(url));
		assertTrue(Location.isRawURL(url));
		
		url="http://asdkfjlsdf:123";
		assertFalse(Location.isU6URL(url));
		assertTrue(Location.isRawURL(url));
		
		url="https://asdkfjlsdf:123";
		assertFalse(Location.isU6URL(url));
		assertTrue(Location.isRawURL(url));
		
		url="FTP:ftp://asdkfjlsdf#/";
		assertTrue(Location.isU6URL(url));
		assertFalse(Location.isRawURL(url));
		
		url="RBYTEIO:ftp://asdkfjlsdf#asd";
		assertTrue(Location.isU6URL(url));
		assertFalse(Location.isRawURL(url));
		
		url="BFT:https://asdkfjlsdf:123#/sdf";
		assertTrue(Location.isU6URL(url));
		assertFalse(Location.isRawURL(url));
		
		url="https://asdkfjlsdf:123#/sdf";
		assertTrue(Location.isU6URL(url));
		assertFalse(Location.isRawURL(url));
	}
	
	public boolean isVerbose(){return true;}
	
	public void verbose(String message){
		System.out.println("[ucc testing] "+message);
	}
	
	public void message(String message){
		System.out.println("[ucc testing] "+message);
	}
	
	public void error(String message, Throwable cause){
		System.err.println("[ucc testing] "+message);
		cause.printStackTrace();
	}
}
