package de.fzj.unicore.ucc.runner;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import org.ggf.schemas.jsdl.x2005.x11.jsdl.OperatingSystemTypeEnumeration.Enum;
import org.junit.Test;
import org.unigrids.x2006.x04.services.tss.ApplicationResourceType;
import org.unigrids.x2006.x04.services.tss.TargetSystemPropertiesDocument;
import org.unigrids.x2006.x04.services.tss.TargetSystemPropertiesDocument.TargetSystemProperties;

import de.fzj.unicore.ucc.runner.ApplicationRequirement;
import de.fzj.unicore.ucc.runner.ExecutionEnvironmentRequirement;
import de.fzj.unicore.ucc.runner.OSRequirement;
import eu.unicore.jsdl.extensions.ExecutionEnvironmentDescriptionDocument.ExecutionEnvironmentDescription;

public class TestRequirements {

	@Test
	public void testExecutionEnvironmentRequirement(){
		TargetSystemProperties tss=getTSSProps();
		ExecutionEnvironmentRequirement req=new ExecutionEnvironmentRequirement("Test",null);
		assertTrue(req.isFulfilled(tss));
		assertFalse(req.getDescription().contains(" v"));
		
		req=new ExecutionEnvironmentRequirement("Test","2.0");
		assertTrue(req.getDescription().contains(" v2.0"));
		
		assertFalse(req.isFulfilled(tss));
		
		req=new ExecutionEnvironmentRequirement("Test","3.2.1");
		assertTrue(req.isFulfilled(tss));
	
		ExecutionEnvironmentRequirement req2=new ExecutionEnvironmentRequirement("Test","3.2.1");
		assertEquals(req,req2);
	
	}
	
	@Test
	public void testOperatingSystemRequirement(){
		TargetSystemProperties tss=getTSSProps();
		OSRequirement req=new OSRequirement(Enum.forString("LINUX"));
		assertTrue(req.getDescription().contains("LINUX"));
		assertTrue(req.isFulfilled(tss));
		
		OSRequirement req2=new OSRequirement(Enum.forString("LINUX"));
		assertEquals(req2,req);
	}
	
	@Test
	public void testApplicationRequirement(){
		TargetSystemProperties tss=getTSSProps();
		ApplicationRequirement req=new ApplicationRequirement("Date",null);
		assertTrue(req.isFulfilled(tss));
		assertFalse(req.getDescription().contains(" v"));
		
		req=new ApplicationRequirement("Date","1.0");
		assertFalse(req.isFulfilled(tss));
		assertTrue(req.getDescription().contains(" v"));
		
		req=new ApplicationRequirement("Date","2.0");
		assertTrue(req.isFulfilled(tss));
		
		ApplicationRequirement req2=new ApplicationRequirement("Date","2.0");
		assertEquals(req,req2);
		
	}
	
	private TargetSystemProperties getTSSProps(){
		TargetSystemPropertiesDocument tssD=TargetSystemPropertiesDocument.Factory.newInstance();
		TargetSystemProperties tss=tssD.addNewTargetSystemProperties();
		
		ExecutionEnvironmentDescription eed=tss.addNewExecutionEnvironmentDescription();
		eed.setName("Test");
		eed.setVersion("3.2.1");
		
		tss.addNewOperatingSystem().addNewOperatingSystemType().setOperatingSystemName(Enum.forString("LINUX"));
		
		ApplicationResourceType app=tss.addNewApplicationResource();
		app.setApplicationName("Date");
		app.setApplicationVersion("2.0");

		return tss;
	}
}
