package de.fzj.unicore.ucc.runner;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import static junit.framework.Assert.*;
import static org.junit.Assert.assertTrue;

import org.junit.Test;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.client.TSSClient;
import de.fzj.unicore.ucc.util.EmbeddedTestBase;
import de.fzj.unicore.wsrflite.admin.ResourceAvailability;
import de.fzj.unicore.wsrflite.utils.WSServerUtilities;
import de.fzj.unicore.wsrflite.xmlbeans.client.RegistryClient;
import de.fzj.unicore.wsrflite.xmlbeans.registry.RegistryCreator;
import de.fzj.unicore.wsrflite.xmlbeans.sg.Registry;

public class TestTargetSystemFinder extends EmbeddedTestBase{

	@Test
	public void testTSFinder()throws Exception{
		connect();
		TargetSystemFinder tsfinder=makeFinder();
		Set<Requirement>requirements=new HashSet<Requirement>();
		requirements.add(new ApplicationRequirement("Date", null));
		TSSClient tss=tsfinder.find(requirements);
		tsfinder.setBlackList("localhost");
		try{
			tsfinder.find(requirements);
			fail();
		}catch(RunnerException re){
			assertEquals(Runner.ERR_NO_SITE,re.getErrorCode());
		}
		//check that empty requirements are accepted
		assertTrue(tsfinder.matches(null, null, null));
		assertTrue(tsfinder.matches(null, new HashSet<Requirement>(), null));
		
		//check handling of unavailable TSF
		tss.destroy();
		toggleAvailable("default_target_system_factory");
		tsfinder.setBlackList(null);
		long start=System.currentTimeMillis();
		try{
			tsfinder.find(requirements);
			fail();
		}catch(RunnerException re){
			assertEquals(Runner.ERR_NO_SITE,re.getErrorCode());
		}
		assertTrue(start-System.currentTimeMillis()<12*1000);
		toggleAvailable("default_target_system_factory");
	}
	
	private TargetSystemFinder makeFinder()throws Exception{
		EndpointReferenceType regEPR=WSServerUtilities.makeEPR(Registry.REGISTRY_SERVICE, 
				RegistryCreator.DEFAULT_REGISTRY_ID, uas.getKernel());
		RegistryClient reg=new RegistryClient(regEPR,uas.getKernel().getClientConfiguration());
		return new TargetSystemFinder(reg,reg.getSecurityConfiguration(),this);
	}
	
	private void toggleAvailable(String id){
		Map<String,String>p=new HashMap<String, String>();
		p.put("resources", id);
		System.out.println(new ResourceAvailability().invoke(p, uas.getKernel()).getMessage());
	}
}
