package de.fzj.unicore.ucc.util;

import static org.junit.Assert.fail;

import java.security.Security;
import java.util.Collections;

import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.junit.AfterClass;
import org.junit.BeforeClass;

import de.fzj.unicore.uas.UAS;

import pl.edu.icm.unicore.uvos.api.Attribute;
import pl.edu.icm.unicore.uvos.api.Element;
import pl.edu.icm.unicore.uvos.api.Group;
import pl.edu.icm.unicore.uvos.api.Identity;
import pl.edu.icm.unicore.uvos.api.IdentityType;
import pl.edu.icm.unicore.uvos.api.Permissions;
import pl.edu.icm.unicore.uvos.api.exceptions.UVOSException;
import pl.edu.icm.unicore.uvos.db.InitDB;
import pl.edu.icm.unicore.uvos.engine.VirtualSecurityContextImpl;
import pl.edu.icm.unicore.uvos.engine.api.UVOSManagementInterface;
import pl.edu.icm.unicore.uvos.server.ComponentContainer;
import pl.edu.icm.unicore.uvos.server.VOServer;
import pl.edu.icm.unicore.uvos.server.idresolver.Authenticator;


/**
 * starts an embedded UAS for testing
 * 
 * @author golbi
 */
public abstract class EmbeddedWithUvos {

	public boolean isVerbose(){return true;}

	protected static VOServer uvos;
	
	private static final String UVOS_CONF = 
			"src/test/resources/uvos/uvosServer.conf";
	private static boolean initDone = false;
	
	protected static UAS uas;
	
	@BeforeClass
	public static void setUp2() {
		Security.removeProvider("BC");
		Security.insertProviderAt(new BouncyCastleProvider(), 10);
		System.out.println("Starting embedded UVOS server, this takes a while...");
		if (initDone)
			return;
		try {
			EmbeddedTestBase.setUp("src/test/resources/uas-saml/uas.config");
			uas=EmbeddedTestBase.uas;
			InitDB.main(new String[]{UVOS_CONF});
			ComponentContainer container = new ComponentContainer(UVOS_CONF);
			uvos = container.getComponent(VOServer.class);
			uvos.startup(container.getContainer());

			addUser(container);
			
			initDone = true;
		} catch (Throwable e) {
			System.err.println("Severe error when starting servers:");
			e.printStackTrace();
			fail();
		}
	}
	
	private static void addUser(ComponentContainer container) throws UVOSException {
		UVOSManagementInterface man = container.getComponent(UVOSManagementInterface.class); 
		Identity testing = new Identity(IdentityType.DN, 
			"CN=Demo User,O=UNICORE,C=EU");
		Authenticator.injectFakeContext(new VirtualSecurityContextImpl(Permissions.ALL_P));
		man.addIdentity(testing);
		Group g = man.addGroup(null, "test-vo");
		man.addToGroup(testing, g);
		man.setAttribute(new Element(g, testing), 
				new Attribute("urn:unicore:attrType:role", 
						Collections.singletonList("user"), null), false);
	}
	
	@AfterClass	
	public static synchronized void tearDown()throws Exception{
		EmbeddedTestBase.tearDown();
		uvos.doShutdown();
	}
}
