package de.fzj.unicore.ucc.util;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.util.HashMap;
import java.util.Map;

import org.json.JSONException;
import org.json.JSONObject;
import org.junit.Test;

public class TestJSONUtil {

	@Test
	public void testSimpleParse()throws JSONException{
		JSONObject o=new JSONObject("{ foo : bar }");
		assertNotNull(o);
		assertEquals("bar", o.getString("foo"));
	}
	
	@Test
	public void testErrorReporting(){
		String json="{ foo : bar \n \n ";
		try{
			new JSONObject(json);
		}catch(JSONException ex){
			String msg=JSONUtil.makeParseErrorMessage(json, ex);
			assertTrue(msg.contains("at line 3"));
		}
	}
	
	@Test
	public void testToMap()throws Exception{
		String json="{ foo : bar, x : y } ";
		JSONObject o=new JSONObject(json);
		Map<String,String>map=JSONUtil.asMap(o);
		assertEquals("y",map.get("x"));
		assertEquals("bar",map.get("foo"));
	}
	
	@Test
	public void testToJSON()throws Exception{
		Map<String,String>map=new HashMap<String, String>();
		map.put("x", "y");
		map.put("foo", "bar");
		JSONObject o=JSONUtil.asJSON(map);
		assertEquals("y",o.getString("x"));
		assertEquals("bar",o.getString("foo"));
	}
}
