package de.fzj.unicore.ucc.util;

import java.util.Arrays;
import java.util.Calendar;
import java.util.List;

import javax.xml.namespace.QName;

import junit.framework.Assert;

import org.junit.Test;
import org.oasisOpen.docs.wsrf.sg2.EntryType;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.ucc.MessageWriter;
import de.fzj.unicore.ucc.helpers.DefaultMessageWriter;
import de.fzj.unicore.wsrflite.xfire.ClientException;
import de.fzj.unicore.wsrflite.xmlbeans.client.IRegistryQuery;

public class TestMultiRegistryQuery {
	
	static boolean invoked;
	
	@Test
	public void testCheckTimeOffset(){
		MultiRegistryQuery mrq=new MultiRegistryQuery(new MyMSG());
		Calendar serverTime=Calendar.getInstance();
		invoked=false;
		Assert.assertTrue(mrq.checkServerTime(Calendar.getInstance(), "test123"));
		Assert.assertFalse(invoked);
		invoked=false;
		serverTime=Calendar.getInstance();
		serverTime.add(Calendar.MINUTE, 11);
		Assert.assertFalse(mrq.checkServerTime(serverTime, "test123"));
		Assert.assertTrue(invoked);
	}

	@Test
	public void testFilterDuplicates()throws Exception{
		MultiRegistryQuery mrq=new MultiRegistryQuery(new DefaultMessageWriter());
		mrq.addRegistry(new MyRegistry());
		mrq.addRegistry(new MyRegistry());
		mrq.setFilterDuplicates(true);
		Assert.assertEquals(1, mrq.listServices(new QName("")).size());
		mrq.setFilterDuplicates(false);
		Assert.assertEquals(2, mrq.listServices(new QName("")).size());
	}
	
	static class MyMSG implements MessageWriter{

		public void verbose(String message) {
		}

		public void message(String message) {
			invoked=true;
		}

		public void error(String message, Throwable cause) {
		}

		public boolean isVerbose() {
			return false;
		}	
	}
	
	static class MyRegistry implements IRegistryQuery{

		public List<EndpointReferenceType> listServices(QName porttype)
				throws Exception {
			String url="http://foo";
			EndpointReferenceType epr=EndpointReferenceType.Factory.newInstance();
			epr.addNewAddress().setStringValue(url);
			return Arrays.asList(new EndpointReferenceType[]{epr});
		}

		public List<EndpointReferenceType> listServices(QName porttype,
				ServiceListFilter acceptFilter) throws Exception {
			return listServices(porttype);
		}

		public List<EndpointReferenceType> listAccessibleServices(QName porttype)
				throws Exception {
			return listServices(porttype);
		}
		public List<EntryType> listEntries() throws Exception {
			return null;
		}

		public String getConnectionStatus() throws ClientException {
			return null;
		}

		public boolean checkConnection() throws ClientException {
			return true;
		}

		@Override
		public boolean checkConnection(int timeout) throws ClientException {
			return true;
		}
		
	}
	
}
