package de.fzj.unicore.ucc.bes;

import java.io.File;

import org.apache.xmlbeans.XmlException;
import org.ggf.schemas.bes.x2006.x08.besFactory.TerminateActivitiesResponseType;
import org.ggf.schemas.bes.x2006.x08.besFactory.TerminateActivityResponseType;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.bes.client.FactoryClient;
import de.fzj.unicore.bes.faults.UnknownActivityIdentifierFault;
import de.fzj.unicore.ucc.actions.ActionBase;
import de.fzj.unicore.ucc.util.Builder;

public class BaseActionState extends ActionBase {
	protected Builder builder;
	protected EndpointReferenceType activityEpr;
	protected FactoryClient factory;

	protected void initBuilder() {
		String arg = null;
		if (getCommandLine().getArgs().length > 1)
			arg = getCommandLine().getArgs()[1];

		if (arg == null) {
			error("Can't find job file.",null);
			endProcessing();
		}

		initFromCmdline(arg);

		buildFactory();

		String eprString = builder.getProperty("activity-epr");
		if (eprString == null) {
			error("Job EPR not found! Maybe <"+arg+"> has not been produced by UCC.",null);
			endProcessing();
		}

		try {
			activityEpr = EndpointReferenceType.Factory.parse(eprString);
			verbose("Job EPR OK.");
		} 
		catch (Exception xe){
			error("Error parsing job EPR",xe);
			endProcessing(1);
		}
	}

	private void initFromCmdline(String arg) {
		try {
			File job = new File(arg);
			if (job.exists()) {
				builder = new Builder(job);
				verbose("Read job from <" + arg + ">");
			} else {
				System.err.println("Couldn't read job file");
				printUsage();
			}
		} catch (Exception e) {
			e.printStackTrace();
			System.err.println("Can't use <" + arg + ">.");
			endProcessing();
		}

	}

	@Override
	public void process() {
		super.process();
		initBuilder();
	}



	protected void terminate() {
		TerminateActivitiesResponseType termType = null;
		try {
			termType = factory.terminateActivity(activityEpr);
		} catch (UnknownActivityIdentifierFault e) {
			error("Couldn't find the specified activity. ", e);
			endProcessing();
		}
		StringBuffer sb = new StringBuffer();
		TerminateActivityResponseType tt = termType.getResponseArray()[0];

		if (tt.getTerminated())
			sb.append("Job is Cancelled.");
		else {
			if (tt.getFault() != null)
				sb.append(termType.getResponseArray()[0].getFault()
						.getFaultstring());
			else
				sb.append("Job will eventually be Cancelled.");
		}

		message(sb.toString());
	}

	protected void buildFactory() {
		String factoryStr = builder.getProperty("factory-epr");
		try {
			EndpointReferenceType factoryEpr = EndpointReferenceType.Factory
					.parse(factoryStr);
			verbose("Accessing Factory at : <"
					+ factoryEpr.getAddress().getStringValue() + ">");
			factory = new FactoryClient(factoryEpr.getAddress()
					.getStringValue(), factoryEpr, securityProperties);
			} catch (XmlException e) {
			error("Couldn't get factory epr", e);
			endProcessing(1);
		} catch (Exception e) {
			error("Couldn't get factory client", e);
			endProcessing(1);

		}

	}

}
