package de.fzj.unicore.ucc.bes;

import javax.xml.namespace.QName;

import org.apache.commons.cli.OptionBuilder;
import org.ggf.schemas.bes.x2006.x08.besFactory.BasicResourceAttributesDocumentType;
import org.ggf.schemas.bes.x2006.x08.besFactory.FactoryResourceAttributesDocumentType;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.OperatingSystemTypeEnumeration;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.bes.BESFactory;
import de.fzj.unicore.bes.client.ActivityClient;
import de.fzj.unicore.bes.client.FactoryClient;
import de.fzj.unicore.ucc.IServiceInfoProvider;
import eu.unicore.util.httpclient.IClientConfiguration;

public class ListFactoryAttributes extends BESActionBase implements BESUCCConstants, IServiceInfoProvider {

	protected ActivityClient activity;
	protected boolean activityFilter;
	private boolean printDetailed;
	
	String job;

	@SuppressWarnings("all")
	@Override
	protected void createOptions() {
		super.createOptions();
		getOptions().addOption(
				OptionBuilder.withLongOpt(OPT_DETAILED_LONG)
						.withArgName("Print Properties in XML").withDescription(
								"Print Properties in XML").isRequired(false)
						.create(OPT_DETAILED));
	}
	
	
	@Override
	public void process() {
		super.process();
		getFactoryAttributes();

	}
	
	protected void getFactoryAttributes(){
		FactoryResourceAttributesDocumentType factoryAttributes = null;
		verbose("Getting Factory Attributes ..");
		try {
			factoryAttributes = factory.getFactoryResourceAttributes().getFactoryResourceAttributesDocument();
		} catch (Exception e) {
			error("Couldn't get resource attributes", e);
			endProcessing();
		}
		
		
		printDetailed = getCommandLine().hasOption(OPT_DETAILED);
		
		
		if(printDetailed){
			message(factoryAttributes.toString());
			return;
		}
		message("\n============ BESFactory Attributes ============\n"); 
		if(factoryAttributes.getCommonName() != null) message("Common Name::: "+factoryAttributes.getCommonName());
		if(factoryAttributes.getLongDescription() != null)message("Description::: "+factoryAttributes.getLongDescription());
		message("Total Number of Activities::: "+factoryAttributes.getTotalNumberOfActivities());
		
		printList("Naming Profile",factoryAttributes.getNamingProfileArray());
		
		printList("Supported Extension",factoryAttributes.getBESExtensionArray());
		
		message("Local Resource Manager::: "+factoryAttributes.getLocalResourceManagerType());
		
		BasicResourceAttributesDocumentType basicResource= factoryAttributes.getBasicResourceAttributesDocument();
		// Printing basic Resource Attributes Document
		
		if(basicResource != null){
			message("\n============ Resource Attributes ============\n");
			if(basicResource.getCPUArchitecture()!=null)message("CPU Architecture::: "+basicResource.getCPUArchitecture().getCPUArchitectureName());
			if(basicResource.getCPUCount()>0)message("CPU Count::: "+basicResource.getCPUCount());
			if(basicResource.getCPUSpeed()>0)message("CPU Speed::: "+basicResource.getCPUSpeed());
		}
		
		
	}
	
	@Override
	public String getDescription() {
		return "provides information about the UNICORE 6 BES Interface";
	}

	@Override
	public String getName() {
		return "bes-list-att";
	}
	
	public void printList(String property, Object[] arr){
		
		for (int i=0; i<arr.length; i++){
			String separator = arr.length>1? "_"+(i+1) : "";
			message(property+separator+"::: "+arr[i]);
		}
		
	}
	
	@Override
	public String getCommandGroup() {
		return BESUCCConstants.BES_COMMAND_GROUP;
	}


	@Override
	public QName getQName() {
		return BESFactory.BES_PORT;
	}


	@Override
	public String getServiceDetails(EndpointReferenceType epr, IClientConfiguration sec) {
		try{
			StringBuilder sb = new StringBuilder();
			
			FactoryClient client = new FactoryClient(epr, sec);
			FactoryResourceAttributesDocumentType resDoc = client.getFactoryResourceAttributes().getFactoryResourceAttributesDocument();
			
			OperatingSystemTypeEnumeration.Enum osType = resDoc.getBasicResourceAttributesDocument().getOperatingSystem().getOperatingSystemType().getOperatingSystemName();
			sb.append(osType);
			
			double cpuCount = getCPUCount(resDoc);
			if(cpuCount > 0){
				sb.append(", ").append(cpuCount).append(" CPU").append((cpuCount > 1)?"s":"");
			}
			
			return sb.toString(); 
		}catch(Exception e){}
		return null;
	}
	
	private double getCPUCount(FactoryResourceAttributesDocumentType resDoc){
		return resDoc.getBasicResourceAttributesDocument().getCPUCount();
	}
	
	
	

	@Override
	public String getServiceName() {
		return "BESFactory";
	}
	
	
}
