package de.fzj.unicore.ucc.workflow;

import java.util.List;

import org.chemomentum.common.ws.ILocationManager;
import org.chemomentum.dataManagement.locationManager.QueryLocationRequestDocument;
import org.chemomentum.dataManagement.locationManager.QueryLocationResponseDocument;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.ucc.MessageWriter;
import de.fzj.unicore.ucc.helpers.Location;
import de.fzj.unicore.wsrflite.xmlbeans.client.IRegistryQuery;
import eu.unicore.security.xfireutil.client.UnicoreXFireClientFactory;
import eu.unicore.util.httpclient.IClientConfiguration;

/**
 * this class can resolve logical filenames
 * 
 * @author schuller
 */	
public class LocationMapperLocation extends Location {

	public LocationMapperLocation(String desc, IRegistryQuery registry, IClientConfiguration sec, MessageWriter msg) {
		super(desc, registry, sec, msg);
	}


	/**
	 * parses the supplied location
	 * @param desc
	 * @return <code>true</code> if location is a valid global file
	 */
	@Override
	protected boolean parseDesc(String desc){
		if(doResolve && desc.startsWith("c9m")){
			return resolveC9MLocation(desc);
		}
		else return super.parseDesc(desc);
	}

	//calls location mapper registed in registry
	private boolean resolveC9MLocation(String desc){
		try{
			List<EndpointReferenceType> available=registry.listServices(ILocationManager.PORT);

			if(available.size()==0){
				msg.message("No location mapping service available!");
			}

			for(EndpointReferenceType epr: available){
				String url=epr.getAddress().getStringValue();
				msg.verbose("Querying location mapping service at "+url);
				//now get location
				ILocationManager lm=new UnicoreXFireClientFactory(sec).createPlainWSProxy(ILocationManager.class,url);
				QueryLocationRequestDocument req=QueryLocationRequestDocument.Factory.newInstance();
				req.addNewQueryLocationRequest().addNewInput().setLogicalName(desc);
				QueryLocationResponseDocument res=lm.queryLocation(req);
				if(res.getQueryLocationResponse().getFaultsOccurred()){
					msg.verbose("Could not resolve location, result "+res);
				}
				else{
					String[]locs=res.getQueryLocationResponse().getResultArray(0).getPhysicalLocationArray();
					if(locs.length>0){
						String l=locs[0];
						msg.message("Resolved logical location as: "+l);
						//now let super class resolve the physical location
						return super.parseDesc(l);
					}
					else return false;
				}
			}
		}catch(Exception e){
			msg.error("Can't resolve location using available location mappers.", e);
		}

		msg.message("No physical location found!");
		return false;
	}

}
